@@ -8,13 +8,13 @@ import (
 	"strings"
 	"time"
 
-	"go.mongodb.org/mongo-driver/mongo"
 	"golang.org/x/mod/semver"
 	"gopkg.in/yaml.v2"
 
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
 	"github.com/percona/percona-backup-mongodb/pbm/backup"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
+	"github.com/percona/percona-backup-mongodb/pbm/config"
 	"github.com/percona/percona-backup-mongodb/pbm/connect"
 	"github.com/percona/percona-backup-mongodb/pbm/ctrl"
 	"github.com/percona/percona-backup-mongodb/pbm/defs"
@@ -33,8 +33,10 @@ type backupOpts struct {
 	base             bool
 	compression      string
 	compressionLevel []int
+	profile          string
 	ns               string
 	wait             bool
+	waitTime         time.Duration
 	externList       bool
 }
 
@@ -44,7 +46,7 @@ type backupOut struct {
 }
 
 func (b backupOut) String() string {
-	return fmt.Sprintf("Backup '%s' to remote store '%s' has started", b.Name, b.Storage)
+	return fmt.Sprintf("Backup '%s' to remote store '%s'", b.Name, b.Storage)
 }
 
 type externBcpOut struct {
@@ -81,7 +83,7 @@ type descBcp struct {
 func runBackup(
 	ctx context.Context,
 	conn connect.Client,
-	pbm sdk.Client,
+	pbm *sdk.Client,
 	b *backupOpts,
 	outf outFormat,
 ) (fmt.Stringer, error) {
@@ -102,7 +104,7 @@ func runBackup(
 
 	if err := checkConcurrentOp(ctx, conn); err != nil {
 		// PITR slicing can be run along with the backup start - agents will resolve it.
-		var e concurentOpError
+		var e *concurentOpError
 		if !errors.As(err, &e) {
 			return nil, err
 		}
@@ -111,12 +113,15 @@ func runBackup(
 		}
 	}
 
-	cfg, err := pbm.GetConfig(ctx)
+	cfg, err := config.GetProfiledConfig(ctx, conn, b.profile)
 	if err != nil {
-		if errors.Is(err, mongo.ErrNoDocuments) {
-			return nil, errors.New("no store set. Set remote store with <pbm store set>")
+		if errors.Is(err, config.ErrMissedConfig) {
+			return nil, errors.New("no config set. Set config with <pbm config>")
 		}
-		return nil, errors.Wrap(err, "get remote-store")
+		if errors.Is(err, config.ErrMissedConfigProfile) {
+			return nil, errors.Errorf("profile %q is not found", b.profile)
+		}
+		return nil, errors.Wrap(err, "get config")
 	}
 
 	compression := cfg.Backup.Compression
@@ -139,26 +144,27 @@ func runBackup(
 			Compression:      compression,
 			CompressionLevel: level,
 			Filelist:         b.externList,
+			Profile:          b.profile,
 		},
 	})
 	if err != nil {
 		return nil, errors.Wrap(err, "send command")
 	}
 
-	if outf != outText {
-		return backupOut{b.name, cfg.Storage.Path()}, nil
-	}
+	showProgress := outf == outText
 
-	fmt.Printf("Starting backup '%s'", b.name)
+	if showProgress {
+		fmt.Printf("Starting backup '%s'", b.name)
+	}
 	startCtx, cancel := context.WithTimeout(ctx, cfg.Backup.Timeouts.StartingStatus())
 	defer cancel()
-	err = waitForBcpStatus(startCtx, conn, b.name)
+	err = waitForBcpStatus(startCtx, conn, b.name, showProgress)
 	if err != nil {
 		return nil, err
 	}
 
 	if b.typ == string(defs.ExternalBackup) {
-		s, err := waitBackup(ctx, conn, b.name, defs.StatusCopyReady)
+		s, err := waitBackup(ctx, conn, b.name, defs.StatusCopyReady, showProgress)
 		if err != nil {
 			return nil, errors.Wrap(err, "waiting for the `copyReady` status")
 		}
@@ -186,12 +192,25 @@ func runBackup(
 	}
 
 	if b.wait {
-		fmt.Printf("\nWaiting for '%s' backup...", b.name)
-		s, err := waitBackup(ctx, conn, b.name, defs.StatusDone)
-		if s != nil {
+		if b.waitTime > time.Second {
+			var cancel context.CancelFunc
+			ctx, cancel = context.WithTimeout(ctx, b.waitTime)
+			defer cancel()
+		}
+
+		if showProgress {
+			fmt.Printf("\nWaiting for '%s' backup...", b.name)
+		}
+		s, err := waitBackup(ctx, conn, b.name, defs.StatusDone, showProgress)
+		if s != nil && showProgress {
 			fmt.Printf(" %s\n", *s)
 		}
-		return outMsg{}, err
+		if errors.Is(err, context.DeadlineExceeded) {
+			err = errWaitTimeout
+		}
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	return backupOut{b.name, cfg.Storage.Path()}, nil
@@ -213,7 +232,13 @@ func runFinishBcp(ctx context.Context, conn connect.Client, bcp string) (fmt.Str
 		backup.ChangeBackupState(conn, bcp, defs.StatusCopyDone, "")
 }
 
-func waitBackup(ctx context.Context, conn connect.Client, name string, status defs.Status) (*defs.Status, error) {
+func waitBackup(
+	ctx context.Context,
+	conn connect.Client,
+	name string,
+	status defs.Status,
+	showProgress bool,
+) (*defs.Status, error) {
 	t := time.NewTicker(time.Second)
 	defer t.Stop()
 
@@ -235,11 +260,13 @@ func waitBackup(ctx context.Context, conn connect.Client, name string, status de
 			}
 		}
 
-		fmt.Print(".")
+		if showProgress {
+			fmt.Print(".")
+		}
 	}
 }
 
-func waitForBcpStatus(ctx context.Context, conn connect.Client, bcpName string) error {
+func waitForBcpStatus(ctx context.Context, conn connect.Client, bcpName string, showProgress bool) error {
 	tk := time.NewTicker(time.Second)
 	defer tk.Stop()
 
@@ -247,7 +274,9 @@ func waitForBcpStatus(ctx context.Context, conn connect.Client, bcpName string)
 	for {
 		select {
 		case <-tk.C:
-			fmt.Print(".")
+			if showProgress {
+				fmt.Print(".")
+			}
 			var err error
 			bmeta, err = backup.NewDBManager(conn).GetBackupByName(ctx, bcpName)
 			if errors.Is(err, errors.ErrNotFound) {
@@ -304,6 +333,7 @@ type bcpDesc struct {
 	Status             defs.Status     `json:"status" yaml:"status"`
 	Size               int64           `json:"size" yaml:"-"`
 	HSize              string          `json:"size_h" yaml:"size_h"`
+	StorageName        string          `json:"storage_name,omitempty" yaml:"storage_name,omitempty"`
 	Err                *string         `json:"error,omitempty" yaml:"error,omitempty"`
 	Replsets           []bcpReplDesc   `json:"replsets" yaml:"replsets"`
 }
@@ -348,24 +378,24 @@ func byteCountIEC(b int64) string {
 	return fmt.Sprintf("%.1f %ciB", float64(b)/float64(div), "KMGTPE"[exp])
 }
 
-func describeBackup(ctx context.Context, conn connect.Client, pbm sdk.Client, b *descBcp) (fmt.Stringer, error) {
-	opts := sdk.GetBackupByNameOptions{}
-	bcp, err := pbm.GetBackupByName(ctx, b.name, opts)
+func describeBackup(ctx context.Context, pbm *sdk.Client, b *descBcp) (fmt.Stringer, error) {
+	bcp, err := pbm.GetBackupByName(ctx, b.name, sdk.GetBackupByNameOptions{})
 	if err != nil {
-		return nil, err
+		return nil, errors.Wrap(err, "get backup meta")
 	}
 
 	var stg storage.Storage
-	if b.coll {
-		l := log.FromContext(ctx).NewDefaultEvent()
-		stg, err = util.GetStorage(ctx, conn, l)
+	if b.coll || bcp.Size == 0 {
+		// to read backed up collection names
+		// or calculate size of files for legacy backups
+		stg, err = util.StorageFromConfig(&bcp.Store.StorageConf, log.LogEventFromContext(ctx))
 		if err != nil {
 			return nil, errors.Wrap(err, "get storage")
 		}
 
-		_, err = stg.FileStat(defs.StorInitFile)
-		if err != nil {
-			return nil, errors.Wrap(err, "check storage access")
+		err = storage.HasReadAccess(ctx, stg)
+		if err != nil && !errors.Is(err, storage.ErrUninitialized) {
+			return nil, errors.Wrap(err, "check read access")
 		}
 	}
 
@@ -384,6 +414,7 @@ func describeBackup(ctx context.Context, conn connect.Client, pbm sdk.Client, b
 		Status:             bcp.Status,
 		Size:               bcp.Size,
 		HSize:              byteCountIEC(bcp.Size),
+		StorageName:        bcp.Store.Name,
 	}
 	if bcp.Err != "" {
 		rv.Err = &bcp.Err
@@ -392,12 +423,6 @@ func describeBackup(ctx context.Context, conn connect.Client, pbm sdk.Client, b
 	if bcp.Size == 0 {
 		switch bcp.Status {
 		case defs.StatusDone, defs.StatusCancelled, defs.StatusError:
-			l := log.FromContext(ctx).NewDefaultEvent()
-			stg, err := util.GetStorage(ctx, conn, l)
-			if err != nil {
-				return nil, errors.Wrap(err, "get storage")
-			}
-
 			rv.Size, err = getLegacySnapshotSize(bcp, stg)
 			if errors.Is(err, errMissedFile) && bcp.Status != defs.StatusDone {
 				// canceled/failed backup can be incomplete. ignore
@@ -460,7 +485,7 @@ func describeBackup(ctx context.Context, conn connect.Client, pbm sdk.Client, b
 // in given `bcps`.
 func bcpsMatchCluster(
 	bcps []backup.BackupMeta,
-	ver,
+	ver string,
 	fcv string,
 	shards []topo.Shard,
 	confsrv string,
@@ -477,7 +502,14 @@ func bcpsMatchCluster(
 	}
 }
 
-func bcpMatchCluster(bcp *backup.BackupMeta, ver, fcv string, shards map[string]bool, mapRS, mapRevRS util.RSMapFunc) {
+func bcpMatchCluster(
+	bcp *backup.BackupMeta,
+	ver string,
+	fcv string,
+	shards map[string]bool,
+	mapRS util.RSMapFunc,
+	mapRevRS util.RSMapFunc,
+) {
 	if bcp.Status != defs.StatusDone {
 		return
 	}
