@@ -27,6 +27,7 @@ type Backup struct {
 	leadConn            connect.Client
 	nodeConn            *mongo.Client
 	brief               topo.NodeBrief
+	config              *config.Config
 	mongoVersion        string
 	typ                 defs.BackupType
 	incrBase            bool
@@ -73,6 +74,10 @@ func NewIncremental(leadConn connect.Client, conn *mongo.Client, brief topo.Node
 	}
 }
 
+func (b *Backup) SetConfig(cfg *config.Config) {
+	b.config = cfg
+}
+
 func (b *Backup) SetMongoVersion(v string) {
 	b.mongoVersion = v
 }
@@ -87,7 +92,7 @@ func (b *Backup) SetSlicerInterval(d time.Duration) {
 
 func (b *Backup) SlicerInterval() time.Duration {
 	if b.oplogSlicerInterval == 0 {
-		return defs.PITRdefaultSpan
+		return defs.DefaultPITRInterval
 	}
 
 	return b.oplogSlicerInterval
@@ -97,10 +102,7 @@ func (b *Backup) Init(
 	ctx context.Context,
 	bcp *ctrl.BackupCmd,
 	opid ctrl.OPID,
-	inf *topo.NodeInfo,
-	store config.StorageConf,
 	balancer topo.BalancerMode,
-	l log.LogEvent,
 ) error {
 	ts, err := topo.GetClusterTime(ctx, b.leadConn)
 	if err != nil {
@@ -113,10 +115,14 @@ func (b *Backup) Init(
 		Name:        bcp.Name,
 		Namespaces:  bcp.Namespaces,
 		Compression: bcp.Compression,
-		Store:       store,
-		StartTS:     time.Now().Unix(),
-		Status:      defs.StatusStarting,
-		Replsets:    []BackupReplset{},
+		Store: Storage{
+			Name:        b.config.Name,
+			IsProfile:   b.config.IsProfile,
+			StorageConf: b.config.Storage,
+		},
+		StartTS:  time.Now().Unix(),
+		Status:   defs.StatusStarting,
+		Replsets: []BackupReplset{},
 		// the driver (mongo?) sets TS to the current wall clock if TS was 0, so have to init with 1
 		LastWriteTS: primitive.Timestamp{T: 1, I: 1},
 		// the driver (mongo?) sets TS to the current wall clock if TS was 0, so have to init with 1
@@ -128,23 +134,13 @@ func (b *Backup) Init(
 		Hb:             ts,
 	}
 
-	cfg, err := config.GetConfig(ctx, b.leadConn)
-	if err != nil {
-		return errors.Wrap(err, "unable to get PBM config settings")
-	}
-	_, err = util.StorageFromConfig(cfg.Storage, l)
-	if errors.Is(err, util.ErrStorageUndefined) {
-		return errors.New("backups cannot be saved because PBM storage configuration hasn't been set yet")
-	}
-	meta.Store = cfg.Storage
-
 	fcv, err := version.GetFCV(ctx, b.nodeConn)
 	if err != nil {
 		return errors.Wrap(err, "get featureCompatibilityVersion")
 	}
 	meta.FCV = fcv
 
-	if inf.IsSharded() {
+	if b.brief.Sharded {
 		ss, err := topo.ClusterMembers(ctx, b.leadConn.MongoClient())
 		if err != nil {
 			return errors.Wrap(err, "get shards")
@@ -193,7 +189,7 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 		rsMeta.IsConfigSvr = &v
 	}
 
-	stg, err := util.GetStorage(ctx, b.leadConn, l)
+	stg, err := util.StorageFromConfig(&b.config.Storage, l)
 	if err != nil {
 		return errors.Wrap(err, "unable to get PBM storage configuration settings")
 	}
@@ -266,7 +262,23 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 				}
 			}
 		}()
+	}
 
+	err = storage.HasReadAccess(ctx, stg)
+	if err != nil {
+		if !errors.Is(err, storage.ErrUninitialized) {
+			return errors.Wrap(err, "check read access")
+		}
+
+		if inf.IsLeader() {
+			err = util.Initialize(ctx, stg)
+			if err != nil {
+				return errors.Wrap(err, "init storage")
+			}
+		}
+	}
+
+	if inf.IsSharded() && inf.IsLeader() {
 		if bcpm.BalancerStatus == topo.BalancerModeOn {
 			err = topo.SetBalancerStatus(ctx, b.leadConn, topo.BalancerModeOff)
 			if err != nil {
@@ -281,7 +293,7 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 
 	// Waiting for StatusStarting to move further.
 	// In case some preparations has to be done before backup.
-	err = b.waitForStatus(ctx, bcp.Name, defs.StatusStarting, ref(b.timeouts.StartingStatus()))
+	err = b.waitForStatus(ctx, bcp.Name, defs.StatusStarting, util.Ref(b.timeouts.StartingStatus()))
 	if err != nil {
 		return errors.Wrap(err, "waiting for start")
 	}
@@ -365,7 +377,7 @@ Loop:
 		}
 	}
 
-	if bs != nil {
+	if bs == nil {
 		return topo.BalancerMode("")
 	}
 
@@ -732,7 +744,3 @@ func condAll[T any, Cond func(*T) bool](ts []T, ok Cond) bool {
 
 	return true
 }
-
-func ref[T any](v T) *T {
-	return &v
-}
