@@ -1,14 +1,14 @@
 package snapshot
 
 import (
+	"context"
 	"io"
-	"sync"
+	"strings"
 	"sync/atomic"
 
-	"github.com/pkg/errors"
-
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
+	"github.com/percona/percona-backup-mongodb/pbm/errors"
 )
 
 type UploadDumpOptions struct {
@@ -27,32 +27,41 @@ type UploadDumpOptions struct {
 
 type UploadFunc func(ns, ext string, r io.Reader) error
 
-func UploadDump(wt io.WriterTo, upload UploadFunc, opts UploadDumpOptions) (int64, error) {
-	wg := sync.WaitGroup{}
+func UploadDump(ctx context.Context, wt io.WriterTo, upload UploadFunc, opts UploadDumpOptions) (int64, error) {
 	pr, pw := io.Pipe()
 	size := int64(0)
 
+	ctx, cancel := context.WithCancel(ctx)
+	defer cancel()
+
+	go func() {
+		<-ctx.Done()
+		pw.CloseWithError(ctx.Err())
+	}()
+
 	go func() {
 		_, err := wt.WriteTo(pw)
-		pw.CloseWithError(errors.WithMessage(err, "write to"))
+		pw.CloseWithError(errors.Wrap(err, "write to"))
 	}()
 
 	newWriter := func(ns string) (io.WriteCloser, error) {
 		pr, pw := io.Pipe()
 
-		wg.Add(1)
+		done := make(chan error)
 		go func() {
-			defer wg.Done()
+			defer close(done)
 
 			ext := ""
 			if ns != archive.MetaFile {
-				ext += opts.Compression.Suffix()
+				ext = opts.Compression.Suffix()
 			}
 
 			rc := &readCounter{r: pr}
 			err := upload(ns, ext, rc)
 			if err != nil {
-				pr.CloseWithError(errors.WithMessagef(err, "upload: %q", ns))
+				err = errors.Wrapf(err, "upload: %q", ns)
+				pr.CloseWithError(err)
+				done <- err
 			}
 
 			atomic.AddInt64(&size, rc.n)
@@ -63,13 +72,24 @@ func UploadDump(wt io.WriterTo, upload UploadFunc, opts UploadDumpOptions) (int6
 		}
 
 		w, err := compress.Compress(pw, opts.Compression, opts.CompressionLevel)
-		dwc := io.WriteCloser(&delegatedWriteCloser{w, pw})
-		return dwc, errors.WithMessagef(err, "create compressor: %q", ns)
+		dwc := io.WriteCloser(&delegatedWriteCloser{w, funcCloser(func() error {
+			err0 := w.Close()
+			err1 := pw.Close()
+			err2 := <-done
+			return errors.Join(err0, err1, err2)
+		})})
+		return dwc, errors.Wrapf(err, "create compressor: %q", ns)
 	}
 
 	err := archive.Decompose(pr, newWriter, opts.NSFilter, opts.DocFilter)
-	wg.Wait()
-	return size, errors.WithMessage(err, "decompose")
+	if err != nil && !errors.Is(err, context.Canceled) {
+		// note: mongo-tools errors cannot be used for errors.Is()
+		if strings.Contains(err.Error(), "( context canceled )") {
+			err = context.Canceled
+		}
+	}
+
+	return size, errors.Wrap(err, "decompose")
 }
 
 type DownloadFunc func(filename string) (io.ReadCloser, error)
@@ -89,7 +109,7 @@ func DownloadDump(
 
 			r, err := download(ns)
 			if err != nil {
-				return nil, errors.WithMessagef(err, "download: %q", ns)
+				return nil, errors.Wrapf(err, "download: %q", ns)
 			}
 
 			if ns == archive.MetaFile {
@@ -97,11 +117,11 @@ func DownloadDump(
 			}
 
 			r, err = compress.Decompress(r, compression)
-			return r, errors.WithMessagef(err, "create decompressor: %q", ns)
+			return r, errors.Wrapf(err, "create decompressor: %q", ns)
 		}
 
 		err := archive.Compose(pw, match, newReader)
-		pw.CloseWithError(errors.WithMessage(err, "compose"))
+		pw.CloseWithError(errors.Wrap(err, "compose"))
 	}()
 
 	return pr, nil
@@ -118,8 +138,14 @@ func (c *readCounter) Read(p []byte) (int, error) {
 	return n, err
 }
 
+type funcCloser func() error
+
+func (f funcCloser) Close() error {
+	return f()
+}
+
 type delegatedWriteCloser struct {
-	w io.WriteCloser
+	w io.Writer
 	c io.Closer
 }
 
@@ -128,17 +154,5 @@ func (d *delegatedWriteCloser) Write(b []byte) (int, error) {
 }
 
 func (d *delegatedWriteCloser) Close() error {
-	we := d.w.Close()
-	ce := d.c.Close()
-
-	switch {
-	case we != nil && ce != nil:
-		return errors.Errorf("writer: %s; closer: %s", we.Error(), ce.Error())
-	case we != nil:
-		return errors.Errorf("writer: %s", we.Error())
-	case ce != nil:
-		return errors.Errorf("closer: %s", ce.Error())
-	}
-
-	return nil
+	return d.c.Close()
 }
