@@ -2,28 +2,31 @@ package storage
 
 import (
 	"context"
+	"fmt"
 	"io"
 
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	"github.com/percona/percona-backup-mongodb/pbm/defs"
 	"github.com/percona/percona-backup-mongodb/pbm/errors"
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 )
 
 var (
 	// ErrNotExist is an error for file doesn't exists on storage
-	ErrNotExist = errors.New("no such file")
-	ErrEmpty    = errors.New("file is empty")
+	ErrNotExist      = errors.New("no such file")
+	ErrEmpty         = errors.New("file is empty")
+	ErrUninitialized = errors.New("uninitialized")
 )
 
 // Type represents a type of the destination storage for backups
 type Type string
 
 const (
-	Undef      Type = ""
+	Undefined  Type = ""
 	S3         Type = "s3"
 	Azure      Type = "azure"
 	Filesystem Type = "filesystem"
-	BlackHole  Type = "blackhole"
+	Blackhole  Type = "blackhole"
 )
 
 type FileInfo struct {
@@ -56,19 +59,70 @@ func ParseType(s string) Type {
 		return Azure
 	case string(Filesystem):
 		return Filesystem
-	case string(BlackHole):
-		return BlackHole
+	case string(Blackhole):
+		return Blackhole
 	default:
-		return Undef
+		return Undefined
 	}
 }
 
-// HasReadAccess checks if the storage has read access to the specified file.
-// It returns true if read access is available, otherwise it returns false.
-// If an error occurs during the check, it returns the error.
-func HasReadAccess(ctx context.Context, stg Storage) (bool, error) {
+// IsInitialized checks if there is PBM init file on the storage.
+func IsInitialized(ctx context.Context, stg Storage) (bool, error) {
 	_, err := stg.FileStat(defs.StorInitFile)
-	return err == nil, err
+	if err != nil {
+		if errors.Is(err, ErrNotExist) {
+			return false, nil
+		}
+
+		return false, errors.Wrap(err, "file stat")
+	}
+
+	return true, nil
+}
+
+// HasReadAccess checks if the provided storage allows the reading of file content.
+//
+// It gets the size (stat) and reads the content of the PBM init file.
+//
+// ErrUninitialized is returned if there is no init file.
+func HasReadAccess(ctx context.Context, stg Storage) error {
+	stat, err := stg.FileStat(defs.StorInitFile)
+	if err != nil {
+		if errors.Is(err, ErrNotExist) {
+			return ErrUninitialized
+		}
+
+		return errors.Wrap(err, "file stat")
+	}
+
+	r, err := stg.SourceReader(defs.StorInitFile)
+	if err != nil {
+		return errors.Wrap(err, "open file")
+	}
+	defer func() {
+		err := r.Close()
+		if err != nil {
+			log.LogEventFromContext(ctx).
+				Error("HasReadAccess(): close file: %v", err)
+		}
+	}()
+
+	const MaxCount = 10 // for "v999.99.99"
+	var buf [MaxCount]byte
+	n, err := r.Read(buf[:])
+	if err != nil && !errors.Is(err, io.EOF) {
+		return errors.Wrap(err, "read file")
+	}
+
+	expect := MaxCount
+	if stat.Size < int64(expect) {
+		expect = int(stat.Size)
+	}
+	if n != expect {
+		return errors.Errorf("short read (%d of %d)", n, expect)
+	}
+
+	return nil
 }
 
 // rwError multierror for the read/compress/write-to-store operations set
@@ -93,6 +147,19 @@ func (rwe rwError) Error() string {
 	return r
 }
 
+func (rwe rwError) Unwrap() error {
+	if rwe.read != nil {
+		return rwe.read
+	}
+	if rwe.write != nil {
+		return rwe.write
+	}
+	if rwe.compress != nil {
+		return rwe.compress
+	}
+	return nil
+}
+
 func (rwe rwError) nil() bool {
 	return rwe.read == nil && rwe.compress == nil && rwe.write == nil
 }
@@ -161,3 +228,31 @@ func Upload(
 
 	return n, nil
 }
+
+func PrettySize(size int64) string {
+	const (
+		_          = iota
+		KB float64 = 1 << (10 * iota)
+		MB
+		GB
+		TB
+	)
+
+	if size < 0 {
+		return "unknown"
+	}
+
+	s := float64(size)
+
+	switch {
+	case s >= TB:
+		return fmt.Sprintf("%.2fTB", s/TB)
+	case s >= GB:
+		return fmt.Sprintf("%.2fGB", s/GB)
+	case s >= MB:
+		return fmt.Sprintf("%.2fMB", s/MB)
+	case s >= KB:
+		return fmt.Sprintf("%.2fKB", s/KB)
+	}
+	return fmt.Sprintf("%.2fB", s)
+}
